-------------------------------------------
--- Author: Ketho (EU-Boulderfist)		---
--- License: Public Domain				---
--- Created: 2012.09.16					---
--- Version: 0.5 [2014.11.08]			---
-------------------------------------------
--- Curse			http://www.curse.com/addons/wow/kgratz
--- WoWInterface	http://www.wowinterface.com/downloads/info21641-kGratz.html

local NAME, S = ...
local VERSION = GetAddOnMetadata(NAME, "Version")

kGratz = LibStub("AceAddon-3.0"):NewAddon("kGratz", "AceEvent-3.0", "AceTimer-3.0")
local KGZ = kGratz

local L = S.L
local AchievementBlacklist = S.AchievementBlacklist

local ACR = LibStub("AceConfigRegistry-3.0")
local ACD = LibStub("AceConfigDialog-3.0")

local profile

local crop = ":64:64:4:60:4:60"

	--------------
	--- Legend ---
	--------------

local legend = {
	gz = {"gz", "grats", "gratz"},
	emot = {":)", ";)", "=)", "=]", "8)", ":3", "<3", ":p", ":P", "=D", ";D", "xD", "XD", "^^", "^_^", "^O^", "n_n", "\\o/", "orz"},
}

	--------------
	--- Events ---
	--------------

local events = {
	CHAT_MSG = {
		"CHAT_MSG_PARTY",
		"CHAT_MSG_PARTY_LEADER",
		"CHAT_MSG_INSTANCE_CHAT",
		"CHAT_MSG_INSTANCE_CHAT_LEADER",
		"CHAT_MSG_GUILD",
	},
	"CHAT_MSG_GUILD_ACHIEVEMENT",
}

	--------------------
	--- Raid Targets ---
	--------------------

local RaidTargetIcons

do
	local t = {}
	local RT = "Interface\\TargetingFrame\\UI-RaidTargetingIcon_"
	
	for i = 1, 8 do
		t[i] = "  |T"..RT..i..":16:16:0:3|t  {rt"..i.."}  ".._G["RAID_TARGET_"..i]
	end
	
	RaidTargetIcons = strjoin("\n", unpack(t))
end

	----------------
	--- Defaults ---
	----------------

local defaults = {
	profile = {
		GratzAFK = true,
		FilterAchiev = true,
		GratzDelay = 0,
		GratzCooldown = 300,
		NumRandomGratz = 9,
		GratzMsg = {
			[1] = "<GZ>",
			[4] = "<GZ>!",
			[7] = "<RT> gz <RT>",
			[2] = "gz!",
			[5] = "gz =)",
			[8] = "grats",
			[3] = "<GZ> <EMOT>",
			[6] = "<GZ> <NAME>",
			[9] = "<GZ> <NAME> <EMOT>",
		},
	},
}

	---------------
	--- Options ---
	---------------

local options = {
	type = "group",
	childGroups = "tab",
	name = format("%s |cffADFF2Fv%s|r", NAME, VERSION),
	args = {
		main = {
			type = "group", order = 1,
			name = GAMEOPTIONS_MENU,
			handler = KGZ,
			get = "GetValue",
			set = "SetValue",
			args = {
				inline1 = {
					type = "group", order = 1,
					name = " ",
					inline = true,
					args = {
						GratzAFK = {
							type = "toggle", order = 1,
							width = "full", descStyle = "",
							desc = "|cffFFFF00\""..MARKED_AFK.."\"|r",
							name = "|TInterface\\Icons\\Spell_Nature_Sleep:16:16:1:0"..crop.."|t  "..L.DISABLE_AFK,
						},
						GratzRandom = {
							type = "toggle", order = 4,
							width = "full", descStyle = "",
							name = "|TInterface\\AddOns\\kGratz\\Awesome:16:16:1:1|t  |cffF6ADC6"..L.RANDOM_MESSAGE.."|r",
							set = function(i, v)
								profile.GratzRandom = v
								KGZ:ToggleMsgWidth(v) -- cant call options from within itself
							end,
						},
						AnyAchiev = {
							type = "toggle", order = 2,
							descStyle = "",
							name = "|TInterface\\Icons\\Achievement_Reputation_06:16:16:1:1"..crop.."|t  |cffFFFF00"..ACHIEVEMENTS.."|r",
						},
						FilterAchiev = {
							type = "toggle", order = 3,
							desc = "Internal Blacklist",
							name = "|TInterface\\Icons\\Trade_Engineering:16:16:1:1"..crop.."|t  +"..FILTER,
							disabled = function() return not profile.AnyAchiev end,
						},
						spacing1 = {type = "description", order = 5, name = " "},
						GratzDelay = {
							type = "range", order = 6,
							desc = "0 = |cffFF8000Random|r 2-7 "..strlower(SECONDS),
							name = L.DELAY,
							min = 0, softMin = 0,
							max = 60, softMax = 10,
							step = 0.5,
						},
						GratzCooldown = {
							type = "range", order = 7,
							desc = function() return format(ITEM_COOLDOWN_TOTAL_SEC, profile.GratzCooldown) end,
							name = L.COOLDOWN,
							min = 0, softMin = 0,
							max = 1200, softMax = 600,
							step = 1,
						},
						NumRandomGratz = {
							type = "range", order = 8,
							descStyle = "",
							name = "# |cffF6ADC6"..L.RANDOM_MESSAGE.."|r",
							min = 2, softMin = 2,
							max = 99, softMax = 30,
							step = 1,
							set = function(i, v)
								profile.NumRandomGratz = v
								KGZ:SetupRandomGratz(v)
							end,
							hidden = function() return not profile.GratzRandom end,
						},
					},
				},
				GratzMsg1 = {
					type = "input", order = 10,
					name = " ",
					usage = "\n|cff71D5FFGZ|r, |cff3FBF3FNAME|r, |cffF6ADC6EMOT|r, |cffFF8000RT|r",
					get = function(i) return profile.GratzMsg[1] end,
					set = function(i, v) profile.GratzMsg[1] = (strtrim(v) == "") and defaults.profile.GratzMsg[1] or v end,
				},
				inline2 = {
					type = "group", order = -1,
					name = " ",
					inline = true,
					args = {
						RaidTarget = {
							type = "description", order = 1,
							fontSize = "medium",
							name = RaidTargetIcons,
						},
					},
				},
			},
		},
	},
}

function KGZ:GetValue(i)
	return profile[i[#i]]
end

function KGZ:SetValue(i, v)
	profile[i[#i]] = v
end

local prev

function KGZ:SetupRandomGratz(num)
	local gratz = options.args.main.args
	
	for i = prev or 2, num do
		gratz["GratzMsg"..i] = {
			type = "input", order = i+10,
			name = "",
			get = function(info) return profile.GratzMsg[i] end,
			set = function(info, v)
				if strtrim(v) == "" then
					profile.GratzMsg[i] = defaults.profile.GratzMsg[i]
				else
					profile.GratzMsg[i] = v
				end
			end,
			hidden = function() return not profile.GratzRandom end,
		}
	end
	
	for i = num+1, prev or 0 do
		gratz["GratzMsg"..i] = nil
	end
	prev = num
end

function KGZ:ToggleMsgWidth(v)
	options.args.main.args.GratzMsg1.width = v and "normal" or "full"
end

	----------------------
	--- Initialization ---
	----------------------

function KGZ:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("kGratzDB", defaults, true)
	
	self.db.global.version = VERSION
	
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshDB")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshDB")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshDB")
	self:RefreshDB()
	
	ACR:RegisterOptionsTable("kGratz_Parent", options)
	ACR:RegisterOptionsTable("kGratz_Main", options.args.main)
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	ACR:RegisterOptionsTable("kGratz_Profiles", options.args.profiles)
	
	ACD:AddToBlizOptions("kGratz_Parent", NAME)
	ACD:AddToBlizOptions("kGratz_Main", options.args.main.name, NAME)
	ACD:AddToBlizOptions("kGratz_Profiles", options.args.profiles.name, NAME)
	
	ACD:SetDefaultSize("kGratz_Parent", 610, 520)
	options.args.profiles.order = 2
end

function KGZ:OnEnable()
	for k, v in pairs(events) do
		if type(v) == "table" then
			for _, event in ipairs(v) do
				self:RegisterEvent(event, k)
			end
		else
			self:RegisterEvent(v)
		end
	end
end

function KGZ:OnDisable()
	self:UnregisterAllEvents()
end

function KGZ:RefreshDB()
	profile = self.db.profile
	self:SetupRandomGratz(profile.NumRandomGratz)
	self:ToggleMsgWidth(profile.GratzRandom)
end

	---------------------
	--- Slash Command ---
	---------------------

local enable = {
	["1"] = true,
	on = true,
	enable = true,
	load = true,
}

local disable = {
	["0"] = true,
	off = true,
	disable = true,
	unload = true,
}

for i, v in ipairs({"kgz", "kgratz", "kgrats"}) do
	_G["SLASH_KGRATZ"..i] = "/"..v
end

SlashCmdList.KGRATZ = function(msg, editbox)
	if enable[msg] then
		KGZ:Enable()
		print("|cff33ff99kGratz|r: |cffADFF2F"..VIDEO_OPTIONS_ENABLED.."|r")
	elseif disable[msg] then
		KGZ:Disable()
		print("|cff33ff99kGratz|r: |cffFF2424"..VIDEO_OPTIONS_DISABLED.."|r")
	else
		ACD:Open("kGratz_Parent")
	end
end

	------------------
	--- Auto Gratz ---
	------------------

local cd = {}
local playerName = UnitName("player")

local remap = {
	CHAT_MSG_PARTY = "PARTY",
	CHAT_MSG_PARTY_LEADER = "PARTY",
	CHAT_MSG_INSTANCE_CHAT = "INSTANCE_CHAT",
	CHAT_MSG_INSTANCE_CHAT_LEADER = "INSTANCE_CHAT",
	CHAT_MSG_GUILD = "GUILD",
}

function KGZ:CHAT_MSG(event, msg, name)
	if not strfind(name, playerName) and strfind(msg:lower(), "ding") and strlen(msg) <= 12 and time() > (cd[remap[event]] or 0) then
		name = strmatch(name, "(.+)%-.+") -- remove server name
		cd[remap[event]] = time() + profile.GratzCooldown		
		self:AutoGratz(remap[event], name)
	end
end

function KGZ:CHAT_MSG_GUILD_ACHIEVEMENT(event, msg, name)
	if not strfind(name, playerName) and time() > (cd.GUILD or 0) then
		name = strmatch(name, "(.+)%-.+")
		-- don't gratz if player is afk
		if profile.GratzAFK and UnitIsAFK("player") then return end
		-- not level achievement filter
		if not profile.AnyAchiev and not strfind(msg, LEVEL) then return end
		-- common achievement filter
		if profile.FilterAchiev and AchievementBlacklist[tonumber(msg:match("achievement:(%d+)"))] then return end
		
		cd.GUILD = time() + profile.GratzCooldown
		self:AutoGratz("GUILD", name)
	end
end

local args = {}

function KGZ:AutoGratz(chan, name)
	local args = args
	args.name = (random(2) == 1) and name or name:lower()
	
	args.gz = legend.gz[random(#legend.gz)]
	if random(3) == 1 then -- first character upper case
		args.gz = args.gz:gsub("^%l", strupper)
	end
	
	args.emot = legend.emot[random(#legend.emot)]
	args.rt = "{rt"..random(8).."}"
	
	local msg = profile.GratzRandom and profile.GratzMsg[random(profile.NumRandomGratz)] or profile.GratzMsg[1]
	msg = self:ReplaceArgs(msg, args)
	
	self:ScheduleTimer(function()
		SendChatMessage(msg, chan)
	end, (profile.GratzDelay == 0) and random(20, 70) / 10 or profile.GratzDelay)
end

	---------------
	--- Replace ---
	---------------

function KGZ:ReplaceArgs(msg, args)
	for k in gmatch(msg, "%b<>") do
		-- remove <>, make case insensitive
		local s = strlower(gsub(k, "[<>]", ""))
		
		-- escape special characters
		-- a maybe better alternative to %p is "[%%%.%-%+%?%*%^%$%(%)%[%]%{%}]"
		s = gsub(args[s] or s, "(%p)", "%%%1")
		k = gsub(k, "(%p)", "%%%1")
		
		msg = msg:gsub(k, s)
	end
	wipe(args)
	return msg
end

	---------------------
	--- LibDataBroker ---
	---------------------

local dataobject = {
	type = "launcher",
	icon = "Interface\\Icons\\INV_Misc_Gift_05",
	text = NAME,
	OnClick = function(clickedframe, button)
		if IsModifierKeyDown() then
			SlashCmdList.KGRATZ(KGZ:IsEnabled() and "0" or "1")
		else
			ACD[ACD.OpenFrames.kGratz_Parent and "Close" or "Open"](ACD, "kGratz_Parent")
		end
	end,
	OnTooltipShow = function(tt)
		tt:AddLine("|cffADFF2F"..NAME.."|r")
		tt:AddLine(L.BROKER_CLICK)
		tt:AddLine(L.BROKER_SHIFT_CLICK)
	end,
}

LibStub("LibDataBroker-1.1"):NewDataObject(NAME, dataobject)
